/// <reference types="node" />
import { TransformOptions } from 'stream';
import { FullTransformOptions } from './transforms/types/full-transform-options.type';
import { ErrorTransform } from './errors/error-transform';
import { AsyncTransformFunction } from './transforms/base/simple-async-transform';
import { TransformFunction } from './transforms/base/simple-transform';
import { FilterOptions } from './transforms/utility/filter-transforms';
import { TypedPassThrough } from './transforms/utility/typed-pass-through';
import { PlumberOptions } from './utility/plumber-options.type';
import { AsyncFilterFunction, FilterFunction } from './transforms/types/filter-function.type';
export declare class TransformerBase {
    private defaultTrasformOptions?;
    constructor(defaultTrasformOptions?: TransformOptions | undefined);
    protected mergeOptions<T>(options?: T): TransformOptions & T;
    errorTransform<T>(options?: TransformOptions): ErrorTransform<T>;
}
export declare class Transformer extends TransformerBase {
    readonly async: AsyncTransformer;
    constructor(defaultTrasformOptions?: TransformOptions);
    /**
     *
     * @param length - max length of the created array
     * @param options
     * @returns a transform that creates an array from incoming chunks
     * @example
     * const source = Readable.from([1,2,3]);
     *
     * const arrayJoiner = transformer.arrayJoin<number>(2);
     *
     * source.pipe(arrayJoiner);
     * for await (const elem of arrayJoiner) {
     *     console.log(elem); // [1, 2], [3]
     * }
     */
    arrayJoin<TSource>(length: number, options?: FullTransformOptions<TSource>): import("./transforms/base/simple-transform").SimpleTransform<TSource, TSource[]>;
    /**
     *
     * @param options
     * @returns a transform that pushes elements of array individually
     * @example
     * const source = Readable.from([
     *     [1, 2, 3],
     *     [4, 5],
     * ]);
     *
     * const arraySplitter = transformer.arraySplit<number>();
     *
     * source.pipe(arraySplitter);
     * for await (const elem of arraySplitter) {
     *     console.log(elem); // 1, 2, 3, 4, 5
     * }
     */
    arraySplit<TSource>(options?: FullTransformOptions<TSource[]>): import("./transforms/base/simple-transform").SimpleTransform<TSource[], TSource>;
    /**
     *
     * @param functionToCallOnData
     * @param options
     * @returns a transform that does an action on the chunk without modifying it
     * @example
     * const source = Readable.from([1, 2, 3].map((n) => ({ a: n })));
     *
     * const callOnDataTransform = transformer.callOnData((chunk: { a: number }) => {
     *     chunk.a *= 2;
     *     console.log(chunk.a); // 2, 4, 6
     * });
     *
     * source.pipe(callOnDataTransform);
     * for await (const elem of callOnDataTransform) {
     *     console.log(elem.a); // 1, 2, 3
     * }
     */
    callOnData<TSource>(functionToCallOnData: (data: TSource) => void, options?: FullTransformOptions<TSource>): import("./transforms/base/simple-transform").SimpleTransform<TSource, TSource>;
    /**
     *
     * @param options
     * @returns a tranform that removes all chunks
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const voidTransform = transformer.void();
     *
     * source.pipe(voidTransform);
     * for await (const elem of voidTransform) {
     *     console.log(elem); // no output
     * }
     */
    void<TSource>(options?: FullTransformOptions<TSource>): import("./transforms/base/simple-transform").SimpleTransform<TSource, void>;
    /**
     *
     * @param options
     * @returns a transform that passes data as is
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const passThrough = transformer.passThrough();
     * source.pipe(passThrough);
     * for await (const elem of passThrough) {
     *     console.log(elem); // 1, 2, 3
     * }
     */
    passThrough<T>(options?: FullTransformOptions<T>): TypedPassThrough<T>;
    /**
     *
     * @param filterFunction - function that filters incoming data, passing it only if the filter is true
     * @param options
     * @returns a transform that will remove filtered out chunks
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const filter = transformer.filter((n: number) => n % 2 === 1);
     * source.pipe(filter);
     * for await (const elem of filter) {
     *     console.log(elem); // 1, 3
     * }
     */
    filter<TSource>(filterFunction: FilterFunction<TSource>, options?: FullTransformOptions<TSource> & FilterOptions): import("./transforms/base/simple-transform").SimpleTransform<TSource, TSource>;
    /**
     *
     * @param filterFunction - filter function that controls if the counter is increased
     * @param options
     * @returns a transform that counts the passing chunks, and a getter for the counted amount
     * @example
     * const countOnlyOdds = (n: number) => n % 2 === 0;
     * const readable = Readable.from([1, 2, 3]);
     * const { transform: counterTransform, getCounter } =transformer.counter(countOnlyOdds);
     * readable.pipe(counterTransform);
     * for await (const elem of counterTransform) {
     *     console.log(elem); // 1, 2, 3
     * }
     * console.log(getCounter()); // 1
     *
     */
    counter<TSource>(countFilter?: FilterFunction<TSource>, options?: FullTransformOptions<TSource> & FilterOptions): {
        transform: import("./transforms/base/simple-transform").SimpleTransform<TSource, TSource>;
        getCounter: () => number;
    };
    /**
     *
     * @param filterFunction - function that gets a chunks are returns a boolean
     * @param options
     * @returns 2 transforms, one that gets all the chunks that the filter functions returned true, and one that gets the others
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const fork = transformer.fork((n: number) => n % 2 === 1);
     *
     * fork.filterTrueTransform.on('data', (data) => console.log(`true: ${data}`));
     * fork.filterFalseTransform.on('data', (data) => console.log(`false: ${data}`));
     *
     * source.pipe(fork.filterTrueTransform);
     * source.pipe(fork.filterFalseTransform);
     *
     * await Promise.all([streamEnd(fork.filterTrueTransform), streamEnd(fork.filterFalseTransform)]);
     * // true: 1, false: 2, true: 3
     */
    fork<TSource>(filterFunction: (chunk: TSource) => boolean, options?: FullTransformOptions<TSource>): {
        filterTrueTransform: import("./transforms/base/simple-transform").SimpleTransform<TSource, TSource>;
        filterFalseTransform: import("./transforms/base/simple-transform").SimpleTransform<TSource, TSource>;
    };
    /**
     *
     * @param filterFunction - type guard function
     * @param options
     * @returns a transform that filters chunks and changes the type of the output
     * @example
     * const source = Readable.from(['1', '2', 3]);
     * const takeOnlyStrings = (chunk: unknown): chunk is string => {
     *     return typeof chunk === 'string';
     * };
     * const filter = transformer.typeFilter(takeOnlyStrings);
     * source.pipe(filter);
     * for await (const elem of filter) {
     *     console.log(elem); // 1, 2
     * }
     */
    typeFilter<TSource, TDestination extends TSource>(typeGuardFunction: (chunk: TSource) => chunk is TDestination, options?: FullTransformOptions<TSource>): import("./transforms/base/simple-transform").SimpleTransform<TSource, TDestination>;
    /**
     *
     * @param transformer  - function that changes the chunk it gets
     * @param options
     * @returns a transform that changes the chunk it gets and passes forward
     * @example
     * const source = Readable.from([1, 2, 3]);
     *
     * const transform = transformer.fromFunction((n: number) => n + 1);
     * source.pipe(transform);
     * for await (const elem of transform) {
     *     console.log(elem); // 2, 3, 4
     * }
     */
    fromFunction<TSource, TDestination>(transformer: TransformFunction<TSource, TDestination | undefined>, options?: FullTransformOptions<TSource>): import("./transforms/base/simple-transform").SimpleTransform<TSource, TDestination>;
    /**
     *
     * @param index - index of array to pick
     * @param options
     * @returns a tranform that takes a certain element from a chunk array
     * @example
     * const source = Readable.from([[1, 2, 3], [4, 5, 6], [7, 8], [9]]);
     *
     * const pickThirdElement = transformer.pickElementFromArray(2);
     * source.pipe(pickThirdElement);
     * for await (const elem of pickThirdElement) {
     *     console.log(elem); // 3, 6
     * }
     */
    pickElementFromArray<T>(index: number, options?: FullTransformOptions<T[]>): import("./transforms/base/simple-transform").SimpleTransform<T[], T>;
    /**
     *
     * @param iterable
     * @param options
     * @returns transform like Readable.from but type safe
     * @example
     * const source = transformer.fromIterable([1,2,3]);
     * for await (const elem of source) {
     *     console.log(elem); // 1, 2, 3
     * }
     */
    fromIterable<T>(iterable: Iterable<T>, options?: TransformOptions): TypedPassThrough<T>;
}
export declare class AsyncTransformer extends TransformerBase {
    /**
     *
     * @param functionToCallOnData
     * @param options
     * @returns a transform that does an async action on the chunk without modifying it
     * @example
     * const source = Readable.from([1, 2, 3].map((n) => ({ a: n })));
     *
     * const callOnDataTransform = transformer.async.callOnData(async (chunk: { a: number }) => {
     *     chunk.a *= 2;
     *     console.log(chunk.a); // 2, 4, 6
     * });
     *
     * source.pipe(callOnDataTransform);
     * for await (const elem of callOnDataTransform) {
     *     console.log(elem.a); // 1, 2, 3
     * }
     */
    callOnData<TSource>(functionToCallOnData: (data: TSource) => Promise<void>, options?: FullTransformOptions<TSource>): import("./transforms/base/simple-async-transform").SimpleAsyncTransform<TSource, TSource>;
    /**
     *
     * @param filterFunction - async function that filters incoming data, passing it only if the filter is true
     * @param options
     * @returns a transform that will remove filtered out chunks
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const filter = transformer.async.filter(async (n: number) => n % 2 === 1);
     * source.pipe(filter);
     * for await (const elem of filter) {
     *     console.log(elem); // 1, 3
     * }
     */
    filter<TSource>(filterFunction: AsyncFilterFunction<TSource>, options?: FullTransformOptions<TSource> & FilterOptions): import("./transforms/base/simple-async-transform").SimpleAsyncTransform<TSource, TSource>;
    /**
     *
     * @param filterFunction - async filter function that controls if the counter is increased
     * @param options
     * @returns a transform that counts the passing chunks, and a getter for the counted amount
     * @example
     * const countOnlyOdds = async (n: number) => n % 2 === 0;
     * const readable = Readable.from([1, 2, 3]);
     * const { transform: counterTransform, getCounter } = transformer.async.counter(countOnlyOdds);
     * readable.pipe(counterTransform);
     * for await (const elem of counterTransform) {
     *     console.log(elem); // 1, 2, 3
     * }
     * console.log(getCounter()); // 1
     *
     */
    counter<TSource>(countFilter: AsyncFilterFunction<TSource>, options?: FullTransformOptions<TSource> & FilterOptions): {
        transform: import("./transforms/base/simple-async-transform").SimpleAsyncTransform<TSource, TSource>;
        getCounter: () => number;
    };
    /**
     *
     * @param filterFunction - async function that gets a chunks are returns a boolean
     * @param options
     * @returns 2 transforms, one that gets all the chunks that the filter functions returned true, and one that gets the others
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const fork = transformer.async.fork(async (n: number) => n % 2 === 1);
     *
     * fork.filterTrueTransform.on('data', (data) => console.log(`true: ${data}`));
     * fork.filterFalseTransform.on('data', (data) => console.log(`false: ${data}`));
     *
     * source.pipe(fork.filterTrueTransform);
     * source.pipe(fork.filterFalseTransform);
     *
     * await Promise.all([streamEnd(fork.filterTrueTransform), streamEnd(fork.filterFalseTransform)]);
     * // true: 1, false: 2, true: 3
     */
    fork<TSource>(filterFunction: (chunk: TSource) => Promise<boolean>, options?: FullTransformOptions<TSource>): {
        filterTrueTransform: import("./transforms/base/simple-async-transform").SimpleAsyncTransform<TSource, TSource>;
        filterFalseTransform: import("./transforms/base/simple-async-transform").SimpleAsyncTransform<TSource, TSource>;
    };
    /**
     *
     * @param transformer  - async function that changes the chunk it gets
     * @param options
     * @returns a transform that changes the chunk it gets and passes forward
     * @example
     * const source = Readable.from([1, 2, 3]);
     *
     * const transform = transformer.async.fromFunction(async (n: number) => n + 1);
     * source.pipe(transform);
     * for await (const elem of transform) {
     *     console.log(elem); // 2, 3, 4
     * }
     */
    fromFunction<TSource, TDestination>(transformer: AsyncTransformFunction<TSource, TDestination | undefined>, options?: FullTransformOptions<TSource>): import("./transforms/base/simple-async-transform").SimpleAsyncTransform<TSource, TDestination>;
    /**
     *
     * @deprecated please use fromFunctionConcurrent2
     * @param transformer - async function that changes the chunk it gets
     * @param concurrency - number of concurrent running transformer functions
     * @param options
     * @param plumberOptions - options for piping inner transforms
     * @returns a input and output transform
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const { input, output } = transformer.async.fromFunctionConcurrent(async (n: number) => n + 1, 2);
     *
     * source.pipe(input);
     *
     * for await (const elem of output) {
     *    console.log(elem); // 2, 3, 4
     * }
     */
    fromFunctionConcurrent<TSource, TDestination>(transformer: AsyncTransformFunction<TSource, TDestination | undefined>, concurrency: number, options?: FullTransformOptions<any>, plumberOptions?: PlumberOptions<any>): {
        input: TypedPassThrough<TSource>;
        output: TypedPassThrough<TDestination>;
    };
    /**
     *
     * @param transformer - async function that changes the chunk it gets
     * @param concurrency - number of concurrent running transformer functions
     * @param options
     * @returns a transform that changes the chunk it gets and passes forward
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const concurrentTransform = transformer.async.fromFunctionConcurrent2(async (n: number) => n + 1, 2);
     *
     * source.pipe(concurrentTransform);
     *
     * for await (const elem of concurrentTransform) {
     *    console.log(elem); // 2, 3, 4
     * }
     */
    fromFunctionConcurrent2<TSource, TDestination>(transformer: AsyncTransformFunction<TSource, TDestination | undefined>, concurrency: number, options?: FullTransformOptions<any>): import("./transforms/utility/concurrent-transform").ConcurrentTransform<TSource, TDestination>;
    /**
     *
     * @param iterable
     * @param options
     * @returns transform like Readable.from but type safe
     * @example
     * const source = transformer.async.fromIterable(someAsyncIterator);
     * for await (const elem of source) {
     *     console.log(elem);
     * }
     */
    fromIterable<T>(iterable: AsyncIterable<T>, options?: TransformOptions): TypedPassThrough<T>;
}
export declare const transformer: Transformer;
