"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformer = exports.AsyncTransformer = exports.Transformer = exports.TransformerBase = void 0;
const error_transform_1 = require("./errors/error-transform");
const array_split_transform_1 = require("./transforms/utility/array-split-transform");
const call_on_data_transforms_1 = require("./transforms/utility/call-on-data-transforms");
const void_input_transform_1 = require("./transforms/utility/void-input-transform");
const filter_transforms_1 = require("./transforms/utility/filter-transforms");
const from_function_transforms_1 = require("./transforms/utility/from-function-transforms");
const from_function_concurrent_transform_1 = require("./transforms/utility/from-function-concurrent-transform");
const from_iterable_transform_1 = require("./transforms/utility/from-iterable-transform");
const typed_pass_through_1 = require("./transforms/utility/typed-pass-through");
const pick_element_from_array_transform_1 = require("./transforms/utility/pick-element-from-array-transform");
const array_join_transform_1 = require("./transforms/utility/array-join-transform");
const type_filter_transforms_1 = require("./transforms/utility/type-filter-transforms");
const counter_transform_1 = require("./transforms/utility/counter-transform");
class TransformerBase {
    constructor(defaultTrasformOptions) {
        this.defaultTrasformOptions = defaultTrasformOptions;
    }
    mergeOptions(options) {
        return Object.assign({}, this.defaultTrasformOptions, options);
    }
    errorTransform(options) {
        const finalOptions = this.mergeOptions(options);
        return new error_transform_1.ErrorTransform(finalOptions);
    }
}
exports.TransformerBase = TransformerBase;
class Transformer extends TransformerBase {
    constructor(defaultTrasformOptions) {
        super(defaultTrasformOptions);
        this.async = new AsyncTransformer(defaultTrasformOptions);
    }
    /**
     *
     * @param length - max length of the created array
     * @param options
     * @returns a transform that creates an array from incoming chunks
     * @example
     * const source = Readable.from([1,2,3]);
     *
     * const arrayJoiner = transformer.arrayJoin<number>(2);
     *
     * source.pipe(arrayJoiner);
     * for await (const elem of arrayJoiner) {
     *     console.log(elem); // [1, 2], [3]
     * }
     */
    arrayJoin(length, options) {
        const finalOptions = super.mergeOptions(options);
        return (0, array_join_transform_1.arrayJoinTransform)(length, finalOptions);
    }
    /**
     *
     * @param options
     * @returns a transform that pushes elements of array individually
     * @example
     * const source = Readable.from([
     *     [1, 2, 3],
     *     [4, 5],
     * ]);
     *
     * const arraySplitter = transformer.arraySplit<number>();
     *
     * source.pipe(arraySplitter);
     * for await (const elem of arraySplitter) {
     *     console.log(elem); // 1, 2, 3, 4, 5
     * }
     */
    arraySplit(options) {
        const finalOptions = this.mergeOptions(options);
        return (0, array_split_transform_1.arraySplitTransform)(finalOptions);
    }
    /**
     *
     * @param functionToCallOnData
     * @param options
     * @returns a transform that does an action on the chunk without modifying it
     * @example
     * const source = Readable.from([1, 2, 3].map((n) => ({ a: n })));
     *
     * const callOnDataTransform = transformer.callOnData((chunk: { a: number }) => {
     *     chunk.a *= 2;
     *     console.log(chunk.a); // 2, 4, 6
     * });
     *
     * source.pipe(callOnDataTransform);
     * for await (const elem of callOnDataTransform) {
     *     console.log(elem.a); // 1, 2, 3
     * }
     */
    callOnData(functionToCallOnData, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, call_on_data_transforms_1.callOnDataSyncTransform)(functionToCallOnData, finalOptions);
    }
    /**
     *
     * @param options
     * @returns a tranform that removes all chunks
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const voidTransform = transformer.void();
     *
     * source.pipe(voidTransform);
     * for await (const elem of voidTransform) {
     *     console.log(elem); // no output
     * }
     */
    void(options) {
        const finalOptions = this.mergeOptions(options);
        return (0, void_input_transform_1.voidInputTransform)(finalOptions);
    }
    /**
     *
     * @param options
     * @returns a transform that passes data as is
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const passThrough = transformer.passThrough();
     * source.pipe(passThrough);
     * for await (const elem of passThrough) {
     *     console.log(elem); // 1, 2, 3
     * }
     */
    passThrough(options) {
        const finalOptions = this.mergeOptions(options);
        return new typed_pass_through_1.TypedPassThrough(finalOptions);
    }
    /**
     *
     * @param filterFunction - function that filters incoming data, passing it only if the filter is true
     * @param options
     * @returns a transform that will remove filtered out chunks
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const filter = transformer.filter((n: number) => n % 2 === 1);
     * source.pipe(filter);
     * for await (const elem of filter) {
     *     console.log(elem); // 1, 3
     * }
     */
    filter(filterFunction, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, filter_transforms_1.filterTransform)(filterFunction, finalOptions);
    }
    /**
     *
     * @param filterFunction - filter function that controls if the counter is increased
     * @param options
     * @returns a transform that counts the passing chunks, and a getter for the counted amount
     * @example
     * const countOnlyOdds = (n: number) => n % 2 === 0;
     * const readable = Readable.from([1, 2, 3]);
     * const { transform: counterTransform, getCounter } =transformer.counter(countOnlyOdds);
     * readable.pipe(counterTransform);
     * for await (const elem of counterTransform) {
     *     console.log(elem); // 1, 2, 3
     * }
     * console.log(getCounter()); // 1
     *
     */
    counter(countFilter, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, counter_transform_1.counterTransform)(countFilter, finalOptions);
    }
    /**
     *
     * @param filterFunction - function that gets a chunks are returns a boolean
     * @param options
     * @returns 2 transforms, one that gets all the chunks that the filter functions returned true, and one that gets the others
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const fork = transformer.fork((n: number) => n % 2 === 1);
     *
     * fork.filterTrueTransform.on('data', (data) => console.log(`true: ${data}`));
     * fork.filterFalseTransform.on('data', (data) => console.log(`false: ${data}`));
     *
     * source.pipe(fork.filterTrueTransform);
     * source.pipe(fork.filterFalseTransform);
     *
     * await Promise.all([streamEnd(fork.filterTrueTransform), streamEnd(fork.filterFalseTransform)]);
     * // true: 1, false: 2, true: 3
     */
    fork(filterFunction, options) {
        const finalOptions = this.mergeOptions(options);
        const filterTrueTransform = (0, filter_transforms_1.filterTransform)(filterFunction, finalOptions);
        const filterFalseTransform = (0, filter_transforms_1.filterTransform)((chunk) => !filterFunction(chunk), finalOptions);
        return {
            filterTrueTransform,
            filterFalseTransform,
        };
    }
    /**
     *
     * @param filterFunction - type guard function
     * @param options
     * @returns a transform that filters chunks and changes the type of the output
     * @example
     * const source = Readable.from(['1', '2', 3]);
     * const takeOnlyStrings = (chunk: unknown): chunk is string => {
     *     return typeof chunk === 'string';
     * };
     * const filter = transformer.typeFilter(takeOnlyStrings);
     * source.pipe(filter);
     * for await (const elem of filter) {
     *     console.log(elem); // 1, 2
     * }
     */
    typeFilter(typeGuardFunction, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, type_filter_transforms_1.typeFilterTransform)(typeGuardFunction, finalOptions);
    }
    /**
     *
     * @param transformer  - function that changes the chunk it gets
     * @param options
     * @returns a transform that changes the chunk it gets and passes forward
     * @example
     * const source = Readable.from([1, 2, 3]);
     *
     * const transform = transformer.fromFunction((n: number) => n + 1);
     * source.pipe(transform);
     * for await (const elem of transform) {
     *     console.log(elem); // 2, 3, 4
     * }
     */
    fromFunction(transformer, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, from_function_transforms_1.fromFunctionTransform)(transformer, finalOptions);
    }
    /**
     *
     * @param index - index of array to pick
     * @param options
     * @returns a tranform that takes a certain element from a chunk array
     * @example
     * const source = Readable.from([[1, 2, 3], [4, 5, 6], [7, 8], [9]]);
     *
     * const pickThirdElement = transformer.pickElementFromArray(2);
     * source.pipe(pickThirdElement);
     * for await (const elem of pickThirdElement) {
     *     console.log(elem); // 3, 6
     * }
     */
    pickElementFromArray(index, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, pick_element_from_array_transform_1.pickElementFromArrayTransform)(index, finalOptions);
    }
    /**
     *
     * @param iterable
     * @param options
     * @returns transform like Readable.from but type safe
     * @example
     * const source = transformer.fromIterable([1,2,3]);
     * for await (const elem of source) {
     *     console.log(elem); // 1, 2, 3
     * }
     */
    fromIterable(iterable, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, from_iterable_transform_1.fromIterable)(iterable, finalOptions);
    }
}
exports.Transformer = Transformer;
class AsyncTransformer extends TransformerBase {
    /**
     *
     * @param functionToCallOnData
     * @param options
     * @returns a transform that does an async action on the chunk without modifying it
     * @example
     * const source = Readable.from([1, 2, 3].map((n) => ({ a: n })));
     *
     * const callOnDataTransform = transformer.async.callOnData(async (chunk: { a: number }) => {
     *     chunk.a *= 2;
     *     console.log(chunk.a); // 2, 4, 6
     * });
     *
     * source.pipe(callOnDataTransform);
     * for await (const elem of callOnDataTransform) {
     *     console.log(elem.a); // 1, 2, 3
     * }
     */
    callOnData(functionToCallOnData, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, call_on_data_transforms_1.callOnDataAsyncTransform)(functionToCallOnData, finalOptions);
    }
    /**
     *
     * @param filterFunction - async function that filters incoming data, passing it only if the filter is true
     * @param options
     * @returns a transform that will remove filtered out chunks
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const filter = transformer.async.filter(async (n: number) => n % 2 === 1);
     * source.pipe(filter);
     * for await (const elem of filter) {
     *     console.log(elem); // 1, 3
     * }
     */
    filter(filterFunction, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, filter_transforms_1.asyncFilterTransform)(filterFunction, finalOptions);
    }
    /**
     *
     * @param filterFunction - async filter function that controls if the counter is increased
     * @param options
     * @returns a transform that counts the passing chunks, and a getter for the counted amount
     * @example
     * const countOnlyOdds = async (n: number) => n % 2 === 0;
     * const readable = Readable.from([1, 2, 3]);
     * const { transform: counterTransform, getCounter } = transformer.async.counter(countOnlyOdds);
     * readable.pipe(counterTransform);
     * for await (const elem of counterTransform) {
     *     console.log(elem); // 1, 2, 3
     * }
     * console.log(getCounter()); // 1
     *
     */
    counter(countFilter, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, counter_transform_1.asyncCounterTransform)(countFilter, finalOptions);
    }
    /**
     *
     * @param filterFunction - async function that gets a chunks are returns a boolean
     * @param options
     * @returns 2 transforms, one that gets all the chunks that the filter functions returned true, and one that gets the others
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const fork = transformer.async.fork(async (n: number) => n % 2 === 1);
     *
     * fork.filterTrueTransform.on('data', (data) => console.log(`true: ${data}`));
     * fork.filterFalseTransform.on('data', (data) => console.log(`false: ${data}`));
     *
     * source.pipe(fork.filterTrueTransform);
     * source.pipe(fork.filterFalseTransform);
     *
     * await Promise.all([streamEnd(fork.filterTrueTransform), streamEnd(fork.filterFalseTransform)]);
     * // true: 1, false: 2, true: 3
     */
    fork(filterFunction, options) {
        const finalOptions = this.mergeOptions(options);
        const filterTrueTransform = (0, filter_transforms_1.asyncFilterTransform)(filterFunction, finalOptions);
        const filterFalseTransform = (0, filter_transforms_1.asyncFilterTransform)((chunk) => __awaiter(this, void 0, void 0, function* () { return !(yield filterFunction(chunk)); }), finalOptions);
        return {
            filterTrueTransform,
            filterFalseTransform,
        };
    }
    /**
     *
     * @param transformer  - async function that changes the chunk it gets
     * @param options
     * @returns a transform that changes the chunk it gets and passes forward
     * @example
     * const source = Readable.from([1, 2, 3]);
     *
     * const transform = transformer.async.fromFunction(async (n: number) => n + 1);
     * source.pipe(transform);
     * for await (const elem of transform) {
     *     console.log(elem); // 2, 3, 4
     * }
     */
    fromFunction(transformer, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, from_function_transforms_1.fromAsyncFunctionTransform)(transformer, finalOptions);
    }
    /**
     *
     * @deprecated please use fromFunctionConcurrent2
     * @param transformer - async function that changes the chunk it gets
     * @param concurrency - number of concurrent running transformer functions
     * @param options
     * @param plumberOptions - options for piping inner transforms
     * @returns a input and output transform
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const { input, output } = transformer.async.fromFunctionConcurrent(async (n: number) => n + 1, 2);
     *
     * source.pipe(input);
     *
     * for await (const elem of output) {
     *    console.log(elem); // 2, 3, 4
     * }
     */
    fromFunctionConcurrent(transformer, concurrency, options, plumberOptions) {
        const finalOptions = this.mergeOptions(options);
        return (0, from_function_concurrent_transform_1.fromFunctionConcurrentTransform)(transformer, concurrency, finalOptions, plumberOptions);
    }
    /**
     *
     * @param transformer - async function that changes the chunk it gets
     * @param concurrency - number of concurrent running transformer functions
     * @param options
     * @returns a transform that changes the chunk it gets and passes forward
     * @example
     * const source = Readable.from([1, 2, 3]);
     * const concurrentTransform = transformer.async.fromFunctionConcurrent2(async (n: number) => n + 1, 2);
     *
     * source.pipe(concurrentTransform);
     *
     * for await (const elem of concurrentTransform) {
     *    console.log(elem); // 2, 3, 4
     * }
     */
    fromFunctionConcurrent2(transformer, concurrency, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, from_function_concurrent_transform_1.fromFunctionConcurrentTransform2)(transformer, concurrency, finalOptions);
    }
    /**
     *
     * @param iterable
     * @param options
     * @returns transform like Readable.from but type safe
     * @example
     * const source = transformer.async.fromIterable(someAsyncIterator);
     * for await (const elem of source) {
     *     console.log(elem);
     * }
     */
    fromIterable(iterable, options) {
        const finalOptions = this.mergeOptions(options);
        return (0, from_iterable_transform_1.fromIterable)(iterable, finalOptions);
    }
}
exports.AsyncTransformer = AsyncTransformer;
exports.transformer = new Transformer({ objectMode: true });
//# sourceMappingURL=transformer.js.map